@@ -1,5 +1,5 @@
-## Mon May 27 03:39:04 2019
-## Original file Copyright © 2019 A.C. Guidoum, K. Boukhetala
+## Fri Apr 03 08:54:53 2020
+## Original file Copyright © 2020 A.C. Guidoum, K. Boukhetala
 ## This file is part of the R package Sim.DiffProc
 ## Department of Probabilities & Statistics
 ## Faculty of Mathematics
@@ -26,24 +26,23 @@
 #####
 ##### Milstein1D
  
-.Milstein1D <- function(N =1000,M=1,x0=0,t0=0,T=1,Dt=NULL,drift,diffusion,
+.Milstein1D <- function(N =1000,M=1,x0=0,t0=0,T=1,Dt,drift,diffusion,
                           type=c("ito","str"),...)
                        {
     DSx  <- Simplify(Deriv(diffusion,"x",cache.exp=FALSE))  
     if (type=="ito"){A    <- function(t,x)  eval(drift)}else{
-	driftstr <- eval(Simplify(substitute(expression(e1 - 0.5 * e2 * de2), list(e1 = drift[[1]], e2 = diffusion[[1]], de2 = Deriv(diffusion,"x",cache.exp=FALSE)[[1]]))))
+	driftstr <- eval(Simplify(substitute(expression(e1 + 0.5 * e2 * de2), list(e1 = drift[[1]], e2 = diffusion[[1]], de2 = Deriv(diffusion,"x",cache.exp=FALSE)[[1]]))))
 	A  <- function(t,x) eval(driftstr)
-    # A  <- function(t,x)  eval(drift) - 0.5 * eval(diffusion) * eval(DSx)
 	}
     S  <- function(t,x)  eval(diffusion)
     Sx <- function(t,x)  eval(DSx)
-    if (is.null(Dt)) {
-        Dt <- (T - t0)/N
+    if (missing(Dt)) {
         t <- seq(t0, T, by=Dt)
     } else {
         t <- c(t0, t0 + cumsum(rep(Dt, N)))
 		T <- t[N + 1]
     }
+    Dt <- (T - t0)/N
     W <- matrix(rnorm(N * M, 0, sqrt(Dt)), N, M)
     X <- matrix(x0, N+1, M)
     for (i in 1L:N) {X[i + 1L,] <- X[i,] + A(t[i],X[i,]) * Dt + S(t[i],X[i,]) * W[i,]+0.5* S(t[i],X[i,])*Sx(t[i],X[i,]) *(W[i,]^2 - Dt) }
@@ -57,33 +56,47 @@
 #####
 ##### Milstein2D
 
-.Milstein2D <- function(N =1000,M=1,x0=0,y0=0,t0=0,T=1,Dt=NULL,driftx,diffx,drifty,diffy,
-                          type=c("ito","str"),...)
-                       {					     
+.Milstein2D <- function(N =1000,M=1,x0=0,y0=0,t0=0,T=1,Dt,driftx,diffx,drifty,diffy,
+                          type=c("ito","str"),corr=NULL,...)
+   {	 
     DSx  <- Simplify(Deriv(diffx,"x",cache.exp=FALSE))
     DSy  <- Simplify(Deriv(diffy,"y",cache.exp=FALSE))  
     if (type=="ito"){Ax    <- function(t,x,y) eval(driftx)
                      Ay    <- function(t,x,y) eval(drifty)}else{
-	driftstrx <- eval(Simplify(substitute(expression(e1 - 0.5 * e2 * de2), list(e1 = driftx[[1]], e2 = diffx[[1]], de2 = Deriv(diffx,"x",cache.exp=FALSE)[[1]]))))
-	driftstry <- eval(Simplify(substitute(expression(e1 - 0.5 * e2 * de2), list(e1 = drifty[[1]], e2 = diffy[[1]], de2 = Deriv(diffy,"y",cache.exp=FALSE)[[1]]))))
+		     if (is.null(corr)){
+		       driftstrx <- eval(Simplify(substitute(expression(e1 + 0.5 * e2 * de2), list(e1 = driftx[[1]], e2 = diffx[[1]], de2 = Deriv(diffx,"x",cache.exp=FALSE)[[1]]))))
+		       driftstry <- eval(Simplify(substitute(expression(e1 + 0.5 * e2 * de2), list(e1 = drifty[[1]], e2 = diffy[[1]], de2 = Deriv(diffy,"y",cache.exp=FALSE)[[1]]))))
+		     }else{
+			   C <- stats::cov2cor(corr)
+		       driftstrx <- eval(Simplify(substitute(expression(e1 + 0.5 * e2 * de2 + 0.5 * rho * e3 * de3), list(e1 = driftx[[1]], e2 = diffx[[1]], de2 = Deriv(diffx,"x",cache.exp=FALSE)[[1]],
+			                     e3 = diffy[[1]], de3 = Deriv(diffx,"y",cache.exp=FALSE)[[1]], rho = C[1,2]))))
+		       driftstry <- eval(Simplify(substitute(expression(e1 + 0.5 * e2 * de2 + 0.5 * rho * e3 * de3), list(e1 = drifty[[1]], e2 = diffy[[1]], de2 = Deriv(diffy,"y",cache.exp=FALSE)[[1]],
+                                 e3 = diffx[[1]], de3 = Deriv(diffy,"x",cache.exp=FALSE)[[1]], rho = C[1,2]))))			   
+		     }
     Ax <- function(t,x,y) eval(driftstrx) 
     Ay <- function(t,x,y) eval(driftstry)
-    # Ax  <- function(t,x,y) eval(driftx) - 0.5 * eval(diffx) * eval(DSx)
-    # Ay  <- function(t,x,y) eval(drifty) - 0.5 * eval(diffy) * eval(DSy)
 	}
     Sx  <- function(t,x,y) eval(diffx)
     Sy  <- function(t,x,y) eval(diffy) 
     dSx <- function(t,x,y) eval(DSx)
     dSy <- function(t,x,y) eval(DSy) 
-    if (is.null(Dt)) {
-        Dt <- (T - t0)/N
+    if (missing(Dt)) {
         t <- seq(t0, T, by=Dt)
     } else {
         t <- c(t0, t0 + cumsum(rep(Dt, N)))
 		T <- t[N + 1]
     }
-    W1 <- matrix(rnorm(N * M, 0, sqrt(Dt)), N, M)
-    W2 <- matrix(rnorm(N * M, 0, sqrt(Dt)), N, M)
+    Dt <- (T - t0)/N
+	Z1 <- matrix(rnorm(N * M, 0, sqrt(Dt)), N, M)
+    Z2 <- matrix(rnorm(N * M, 0, sqrt(Dt)), N, M)
+    if (!is.null(corr)){
+        C <- stats::cov2cor(corr)
+	    W1 <- Z1
+        W2 <- C[1,2] * Z1 + sqrt(1-C[1,2]^2) * Z2	
+     }else{
+	    W1 <- Z1
+		W2 <- Z2
+	 }	
     X <- matrix(x0, N+1, M)
     Y <- matrix(y0, N+1, M)	
     for (i in 1L:N) {
@@ -101,8 +114,8 @@
 #####
 ##### Milstein3D
 
-.Milstein3D <- function(N =1000,M=1,x0=0,y0=0,z0=0,t0=0,T=1,Dt=NULL,driftx,diffx,drifty,diffy,
-                     driftz,diffz,type=c("ito","str"),...)
+.Milstein3D <- function(N =1000,M=1,x0=0,y0=0,z0=0,t0=0,T=1,Dt,driftx,diffx,drifty,diffy,
+                     driftz,diffz,type=c("ito","str"),corr=NULL,...)
                        {
     DSx  <- Simplify(Deriv(diffx,"x",cache.exp=FALSE))
     DSy  <- Simplify(Deriv(diffy,"y",cache.exp=FALSE))
@@ -111,15 +124,22 @@
         Ax    <- function(t,x,y,z) eval(driftx)
         Ay    <- function(t,x,y,z) eval(drifty)
         Az    <- function(t,x,y,z) eval(driftz)}else{
-		driftstrx <- eval(Simplify(substitute(expression(e1 - 0.5 * e2 * de2), list(e1 = driftx[[1]], e2 = diffx[[1]], de2 = Deriv(diffx,"x",cache.exp=FALSE)[[1]]))))
-		driftstry <- eval(Simplify(substitute(expression(e1 - 0.5 * e2 * de2), list(e1 = drifty[[1]], e2 = diffy[[1]], de2 = Deriv(diffy,"y",cache.exp=FALSE)[[1]]))))
-	    driftstrz <- eval(Simplify(substitute(expression(e1 - 0.5 * e2 * de2), list(e1 = driftz[[1]], e2 = diffz[[1]], de2 = Deriv(diffz,"z",cache.exp=FALSE)[[1]]))))
+		   	if (is.null(corr)){
+		      driftstrx <- eval(Simplify(substitute(expression(e1 - 0.5 * e2 * de2), list(e1 = driftx[[1]], e2 = diffx[[1]], de2 = Deriv(diffx,"x",cache.exp=FALSE)[[1]]))))
+		      driftstry <- eval(Simplify(substitute(expression(e1 - 0.5 * e2 * de2), list(e1 = drifty[[1]], e2 = diffy[[1]], de2 = Deriv(diffy,"y",cache.exp=FALSE)[[1]]))))
+	          driftstrz <- eval(Simplify(substitute(expression(e1 - 0.5 * e2 * de2), list(e1 = driftz[[1]], e2 = diffz[[1]], de2 = Deriv(diffz,"z",cache.exp=FALSE)[[1]]))))
+			}else{
+			  C <- stats::cov2cor(corr)
+		      driftstrx <- eval(Simplify(substitute(expression(e1 - 0.5 * e2 * de2 - 0.5 * rho1 *e3 * de3 - 0.5 * rho2 *e4 * de4), list(e1 = driftx[[1]], e2 = diffx[[1]], de2 = Deriv(diffx,"x",cache.exp=FALSE)[[1]],
+			   e3 = diffy[[1]], de3 = Deriv(diffx,"y",cache.exp=FALSE)[[1]],e4 = diffz[[1]], de4 = Deriv(diffx,"z",cache.exp=FALSE)[[1]],roh1=C[1,2],rho2=C[1,3]))))
+		      driftstry <- eval(Simplify(substitute(expression(e1 - 0.5 * e2 * de2 - 0.5 * rho1 *e3 * de3 - 0.5 * rho3 *e4 * de4), list(e1 = drifty[[1]], e2 = diffy[[1]], de2 = Deriv(diffy,"y",cache.exp=FALSE)[[1]],
+			   e3 = diffx[[1]], de3 = Deriv(diffy,"x",cache.exp=FALSE)[[1]],e4 = diffz[[1]], de4 = Deriv(diffy,"z",cache.exp=FALSE)[[1]],roh1=C[1,2],rho3=C[2,3]))))			  
+	          driftstrz <- eval(Simplify(substitute(expression(e1 - 0.5 * e2 * de2 - 0.5 * rho2 *e3 * de3 - 0.5 * rho3 *e4 * de4), list(e1 = driftz[[1]], e2 = diffz[[1]], de2 = Deriv(diffz,"z",cache.exp=FALSE)[[1]],
+			   e3 = diffx[[1]], de3 = Deriv(diffz,"x",cache.exp=FALSE)[[1]],e4 = diffy[[1]], de4 = Deriv(diffz,"y",cache.exp=FALSE)[[1]],roh2=C[1,3],rho3=C[2,3]))))			  
+			}
         Ax <- function(t,x,y,z) eval(driftstrx) 
         Ay <- function(t,x,y,z) eval(driftstry)
         Az <- function(t,x,y,z) eval(driftstrz)
-    # Ax <- function(t,x,y,z) eval(driftx) - 0.5 * eval(diffx) * eval(DSx)
-    # Ay <- function(t,x,y,z) eval(drifty) - 0.5 * eval(diffy) * eval(DSy)
-    # Az <- function(t,x,y,z) eval(driftz) - 0.5 * eval(diffz) * eval(DSz)
 	}
     Sx  <- function(t,x,y,z) eval(diffx)
     Sy  <- function(t,x,y,z) eval(diffy)
@@ -127,16 +147,26 @@
     dSx <- function(t,x,y,z) eval(DSx)
     dSy <- function(t,x,y,z) eval(DSy)
     dSz <- function(t,x,y,z) eval(DSz)
-    if (is.null(Dt)) {
-        Dt <- (T - t0)/N
+    if (missing(Dt)) {
         t <- seq(t0, T, by=Dt)
     } else {
         t <- c(t0, t0 + cumsum(rep(Dt, N)))
 		T <- t[N + 1]
-    }
-    W1 <- matrix(rnorm(N * M, 0, sqrt(Dt)), N, M)
-    W2 <- matrix(rnorm(N * M, 0, sqrt(Dt)), N, M)
-	W3 <- matrix(rnorm(N * M, 0, sqrt(Dt)), N, M)
+    } 
+    Dt <- (T - t0)/N
+	Z1 <- matrix(rnorm(N * M, 0, sqrt(Dt)), N, M)
+    Z2 <- matrix(rnorm(N * M, 0, sqrt(Dt)), N, M)
+    Z3 <- matrix(rnorm(N * M, 0, sqrt(Dt)), N, M)
+    if (!is.null(corr)){
+        C <- stats::cov2cor(corr)
+	    W1 <- Z1
+        W2 <- C[1,2] * Z1 + sqrt(1-C[1,2]^2) * Z2	
+		W3 <- C[1,3] * Z1 + ((C[2,3]-C[1,2]*C[1,3])/(sqrt(1-C[1,2]^2))) * Z2 + ( sqrt(1-C[1,3]^2 - ((C[2,3]-C[1,2]*C[1,3])/(sqrt(1-C[1,2]^2)))^2 )  ) * Z3
+     }else{
+	    W1 <- Z1
+		W2 <- Z2
+		W3 <- Z3
+	 }
     X <- matrix(x0, N+1, M)
     Y <- matrix(y0, N+1, M)	
 	Z <- matrix(z0, N+1, M)	
